#!/usr/bin/python
# 

import gtk
import gobject
import gtk.glade
import os
import os.path
import select
import libuser
import dbus

gladedir = "/usr/share/system-config-qemu"

# Hack for dev purposes
if os.path.exists("./system-config-qemu.glade"):
    gladedir = "."

def admin_prompt_callback(prompts, title):
    dialog = gtk.Dialog()
    dialog.set_title(title)
    dialog.set_position(gtk.WIN_POS_CENTER)
    dialog.set_icon(iconPixbuf)
    dialog.add_button(gtk.STOCK_OK, 1)
    dialog.add_button(gtk.STOCK_CANCEL, 0)

    table = gtk.Table(rows = len(prompts), columns = 2)
    dialog.vbox.pack_start(table)

    table.set_row_spacings(4)
    table.set_col_spacings(4)

    ret_list = []
    for i in range(len(prompts)):
        prompt = prompts[i]

        label = gtk.Label(prompt.prompt)
        label.set_alignment(1.0, 0.5)
        table.attach(label, 0, 1, i, i + 1)

        entry = gtk.Entry()
        entry.set_visibility(prompt.visible)
        if prompt.default_value:
            entry.set_text(prompt.default_value)
        table.attach(entry, 1, 2, i, i + 1)

        ret_list.append((prompt, entry))
        
    table.show_all()

    i = dialog.run()
    if i == 1:
        for (prompt, entry) in ret_list:
            prompt.value = entry.get_text()
        dialog.destroy()
        return True

    sys.exit(1)

class Reader:
    def run (self, fileobjs, callback=None, serviceinterval=0.1, readmax=128):
        outputs = {}
        for fileobj in fileobjs:
            outputs[fileobj] = ""
        while len (fileobjs):
            if callback:
                callback ()
            available = select.select (fileobjs, [], [], serviceinterval)
            for fileobj in available[0]:
                text = fileobj.read (readmax)
                if len (text) != 0:
                    outputs[fileobj] += text
                else:
                    #EOF
                    fileobjs.remove (fileobj)

        return outputs

class qemuAdmin:
    def __init__(self):
        self.window = gtk.glade.XML(gladedir + "/system-config-qemu.glade", "config")
        self.window.get_widget("config").show_all()
        
        userList = self.window.get_widget("authorize-users")
        userList.set_model(gtk.ListStore(gobject.TYPE_BOOLEAN, str))
        
        user_tog = gtk.CellRendererToggle()
        user_tog.connect("toggled", self.toggle_user)
        userList.append_column(gtk.TreeViewColumn(None, user_tog, active=0))
        user_text = gtk.CellRendererText()
        userList.append_column(gtk.TreeViewColumn(None, user_text, text=1))

        self.window.signal_autoconnect({
            "on_close_clicked": self.quit,
            "on_apply_clicked": self.save,
            "on_revert_clicked": self.load,
            "on_vlan_enable_toggled": self.toggle_tab_enable,
            "on_accelerator_enable_toggled": self.toggle_tab_enable,
            })
       
        self.ADMIN = libuser.admin(prompt=admin_prompt_callback, prompt_data='Logon Information Required')
        self.group = self.ADMIN.lookupGroupByName("qemu")
        if self.group == None:
            cn = self.ADMIN.initGroup("qemu")
            self.ADMIN.addGroup(cn)
            self.group = self.ADMIN.lookupGroupByName("qemu")

        self.populate_nics()
        self.load()
        self.toggle_tab_enable()


    def quit(self,ignore=None):
        gtk.main_quit()

    def toggle_user(self,data,row):
        userList = self.window.get_widget("authorize-users")
        model = userList.get_model()
        val = model.get_value(model.get_iter(int(row)), 0)
        model.set_value(model.get_iter(int(row)), 0, not val)

    def toggle_tab_enable(self,ignore=None):
        if self.window.get_widget("accelerator-enable").get_active() or self.window.get_widget("vlan-enable").get_active():
            self.window.get_widget("panels").get_nth_page(2).set_sensitive(True)
        else:
            self.window.get_widget("panels").get_nth_page(2).set_sensitive(False)

        if self.window.get_widget("vlan-enable").get_active():
            self.window.get_widget("panels").get_nth_page(1).set_sensitive(True)
        else:
            self.window.get_widget("panels").get_nth_page(1).set_sensitive(False)

    def load(self,ignore=None):
        if self.is_service_on("qvm86"):
            self.window.get_widget("accelerator-enable").set_active(True)
        else:
            self.window.get_widget("accelerator-enable").set_active(False)

        if self.is_service_on("qemu-network"):
            self.window.get_widget("vlan-enable").set_active(True)
        else:
            self.window.get_widget("vlan-enable").set_active(False)


        members = self.group.get(libuser.MEMBERNAME)
        if not members:
            members = []

        userList = self.ADMIN.enumerateUsersFull()
        userList.sort
        accelUsers = self.window.get_widget("authorize-users").get_model()
        accelUsers.clear()
        for user in userList:
            userName = user.get(libuser.USERNAME)[0]
            uidNumber = user.get(libuser.UIDNUMBER)[0]
            if long (uidNumber) > 499L and not (userName == "nfsnobody" and (long (uidNumber) == 65534L or long (uidNumber) == 4294967294L)):
                if userName in members:
                    accelUsers.append([True,userName])
                else:
                    accelUsers.append([False,userName])

        self.load_network()


    def is_service_on(self,name):
        status = self.getstatusoutput("LC_ALL=C /sbin/chkconfig --list " + name)
        if status[0] != 0:
            raise "cannot run /sbin/chkconfig --list " + name

        offset = status[1].find("3:")
        if status[1][offset:offset+4] == "3:on":
            return True
        else:
            return False

    def load_network(self):
        config = file("/etc/sysconfig/qemu-network")

        vlanAddress = "10.20.0.1"
        vlanNetmask = "255.255.255.0"
        vlanNetwork = "10.20.0.0"
        vlanBroadcast = "10.20.0.255"

        dhcpdEnable = True
        dhcpdStartAddress = "10.20.0.100"
        dhcpdEndAddress = "10.20.0.200"

        fwdEnable = True
        fwdDevice = "auto"

        line = config.readline()
        while line != None and line != "":
            if line.startswith("VLAN_IPADDR="):
                vlanAddress = line[12:len(line)-1]
            elif line.startswith("VLAN_NETMASK="):
                vlanNetmask = line[13:len(line)-1]
            elif line.startswith("VLAN_NETWORK="):
                vlanNetwork = line[13:len(line)-1]
            elif line.startswith("VLAN_BROADCAST="):
                vlanBroadcast = line[15:len(line)-1]
            elif line.startswith("DHCPD_ENABLE="):
                if line[13:len(line)-1] == "yes":
                    dhcpdEnable = True
                else:
                    dhcpdEnable = False
            elif line.startswith("DHCP_START_ADDR="):
                dhcpdStartAddress = line[16:len(line)-1]
            elif line.startswith("DHCP_END_ADDR="):
                dhcpdEndAddress = line[14:len(line)-1]
            elif line.startswith("FORWARD_ENABLE="):
                if line[15:len(line)-1] == "yes":
                    fwdEnable = True
                else:
                    fwdEnable = False
            elif line.startswith("FORWARD_DEVICE="):
                fwdDevice = line[15:len(line)-1]
            line = config.readline()
        config.close()

        self.set_ip_address("tun-ipaddr", vlanAddress)
        self.set_ip_address("tun-netmask", vlanNetmask)
        self.set_ip_address("tun-network", vlanNetwork)
        self.set_ip_address("tun-bcast", vlanBroadcast)

        self.window.get_widget("dhcp-enable").set_active(dhcpdEnable)
        self.window.get_widget("dhcp-start-1").set_sensitive(dhcpdEnable)
        self.window.get_widget("dhcp-start-2").set_sensitive(dhcpdEnable)
        self.window.get_widget("dhcp-start-3").set_sensitive(dhcpdEnable)
        self.window.get_widget("dhcp-start-4").set_sensitive(dhcpdEnable)
        self.window.get_widget("dhcp-end-1").set_sensitive(dhcpdEnable)
        self.window.get_widget("dhcp-end-2").set_sensitive(dhcpdEnable)
        self.window.get_widget("dhcp-end-3").set_sensitive(dhcpdEnable)
        self.window.get_widget("dhcp-end-4").set_sensitive(dhcpdEnable)
        self.set_ip_address("dhcp-start", dhcpdStartAddress)
        self.set_ip_address("dhcp-end", dhcpdEndAddress)

        self.window.get_widget("masquerade-enable").set_active(fwdEnable)
        self.window.get_widget("masquerade-device").set_sensitive(fwdEnable)
        
        devices = self.window.get_widget("masquerade-device")
        j = 0
        devices.set_active(-1)
        for row in devices.get_model():
            if row[1] == fwdDevice:
                devices.set_active(j)
                break
            j = j + 1


    def set_ip_address(self, widget, value):
        bits = value.split(".")
        if len(bits) != 4:
            raise "malformed IP addres " + value

        self.window.get_widget(widget + "-1").set_text(bits[0])
        self.window.get_widget(widget + "-2").set_text(bits[1])
        self.window.get_widget(widget + "-3").set_text(bits[2])
        self.window.get_widget(widget + "-4").set_text(bits[3])
        

    def save(self,ignore=None):
        vlanOn = self.is_service_on("qemu-network")

        if vlanOn:
            off = self.getstatusoutput("LC_ALL=C /sbin/service qemu-network stop")

        self.save_network()
        
        vlan = self.window.get_widget("vlan-enable")
        if vlan.get_active():
            off = self.getstatusoutput("LC_ALL=C /sbin/chkconfig qemu-network on")
            off = self.getstatusoutput("LC_ALL=C /sbin/service qemu-network start")
        else:
            off = self.getstatusoutput("LC_ALL=C /sbin/chkconfig qemu-network off")
            off = self.getstatusoutput("LC_ALL=C /sbin/service qemu-network stop")

        accelerator = self.window.get_widget("accelerator-enable")
        if accelerator.get_active():
            off = self.getstatusoutput("LC_ALL=C /sbin/chkconfig qvm86 on")
            off = self.getstatusoutput("LC_ALL=C /sbin/service qvm86 stop")
            off = self.getstatusoutput("LC_ALL=C /sbin/service qvm86 start")
        else:
            off = self.getstatusoutput("LC_ALL=C /sbin/chkconfig qvm86 off")
            off = self.getstatusoutput("LC_ALL=C /sbin/service qvm86 stop")

        newMembers = []
        accelUsers = self.window.get_widget("authorize-users")
        for row in accelUsers.get_model():
            if row[0] == True:
                newMembers.append(row[1])

        self.group.set(libuser.MEMBERNAME, newMembers)
        self.ADMIN.modifyGroup(self.group)

    def save_network(self):
        config = file("/etc/sysconfig/qemu-network.tmp", "w")

        config.write("VLAN_IPADDR=" + self.get_ip_address("tun-ipaddr") + "\n")
        config.write("VLAN_NETMASK=" + self.get_ip_address("tun-netmask") + "\n")
        config.write("VLAN_NETWORK=" + self.get_ip_address("tun-network") + "\n")
        config.write("VLAN_BROADCAST=" + self.get_ip_address("tun-bcast") + "\n")
        config.write("\n")

        if self.window.get_widget("dhcp-enable").get_active():
            config.write("DHCPD_ENABLE=yes\n")
        else:
            config.write("DHCPD_ENABLE=no\n")
        config.write("DHCPD_START_ADDR=" + self.get_ip_address("dhcp-start") + "\n")
        config.write("DHCPD_END_ADDR=" + self.get_ip_address("dhcp-end") + "\n")
        config.write("\n")

        if self.window.get_widget("masquerade-enable").get_active():
            config.write("FORWARD_ENABLE=yes\n")
        else:
            config.write("FORWARD_ENABLE=no\n")

        device = self.window.get_widget("masquerade-device")
        name = "auto"
        if device.get_active() != -1:
            iter = device.get_active_iter()
            model = device.get_model()
            name = model.get_value(iter, 1)

        config.write("FORWARD_DEVICE=" + name + "\n")
        config.close()

        os.unlink("/etc/sysconfig/qemu-network")
        os.rename("/etc/sysconfig/qemu-network.tmp", "/etc/sysconfig/qemu-network")


    def get_ip_address(self, widget):
        bits = ""
        for i in range(4):
            bits = bits + self.window.get_widget(widget + "-" + str(i+1)).get_text()
            if i != 3:
                bits = bits + "."
        return bits

    def populate_nics(self):
        system = dbus.SystemBus()
        hal = dbus.Interface(
            system.get_object("org.freedesktop.Hal", "/org/freedesktop/Hal/Manager"),
            "org.freedesktop.Hal.Manager")

        deviceList = self.window.get_widget("masquerade-device")
        deviceModel = gtk.ListStore(str,str)
        deviceList.set_model(deviceModel)
        
        deviceModel_text = gtk.CellRendererText()
        deviceList.pack_start(deviceModel_text, True)    
        deviceList.add_attribute(deviceModel_text, 'text', 0)

        deviceModel.append(["Automatic (via Network Manager)", "auto"])

        for uuid in hal.FindDeviceByCapability("net"):
            dev = dbus.Interface(system.get_object("org.freedesktop.Hal", uuid), "org.freedesktop.Hal.Device")
            deviceModel.append([dev.GetProperty("net.interface"), dev.GetProperty("net.interface")])


    def uicallback (self):
        while gtk.events_pending ():
            if gtk.__dict__.has_key ("main_iteration"):
                gtk.main_iteration ()
            else:
                gtk.mainiteration () 

    def getstatusoutput(self,cmd):
        pipe = os.popen('{ ' + cmd + '; } 2>/dev/null', 'r')
        output = Reader().run([pipe], self.uicallback)
        text = output[pipe]
        sts = pipe.close ()
        if sts is None: sts = 0
        
        if text[-1:] == '\n': text = text[:-1]
        return sts, text


def main():
    window = qemuAdmin()
    gtk.main()

if __name__ == "__main__":
    main()
